//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|  Sample to show a problem with MT4 build 445.                                                                                                                          |
//|  This indicator takes a long time to be displayed, but with the build 445 this can be infinite.                                                                        |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|  My configuration :                                                                                                                                                    |
//|       Core 2 duo E8400 (3Gz)                                                                                                                                           |
//|       4 GB ram                                                                                                                                                         |
//|       Win 7 64 bits                                                                                                                                                    |
//|                                                                                                                                                                        |
//|       I choose EURUSD M15 chart (with 75000 bars => from november 2009 until now)                                                                                      |
//|       With this configuration, when                                                                                                                                    |
//|         e_period = 70,  it takes 50s                                                                                                                                   |
//|         e_period = 80,  it takes 1min 05s                                                                                                                              |
//|         e_period = 90,  it takes 1min 20s                                                                                                                              |
//|         e_period = 100, it takes 1min 35s                                                                                                                              |
//|         e_period = 105, it takes 1min 50s                                                                                                                              |
//|         e_period = 110, INFINITE ??!? (I stop the MT4 process after 5 minutes)                                                                                         |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|  HOW TO USE INPUT PARAMETERS :                                                                                                                                         |
//|  -----------------------------                                                                                                                                         |
//|                                                                                                                                                                        |
//|      1) e_period [ integer >= 1 ]     =>  50                                                                                                                           |
//|                                                                                                                                                                        |
//|         The filter will be computed on these "n" data.                                                                                                                 |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//|                                                                                                                                                                        |
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#property indicator_chart_window
#property indicator_buffers    1
#property indicator_color1     Crimson
#property indicator_style1     0
#property indicator_width1     2



//************************************************************
// Input parameters
//************************************************************

extern int    e_period       = 50;



//************************************************************
// Constant
//************************************************************
string INDICATOR_NAME = "build445_problem";
string FILENAME       = "build445_problem.mq4";


//************************************************************
// Private vars
//************************************************************

//---- output buffer
double ExtOutputBuffer[];


double g_distance[];







//+================================================================================================================+
//+=== FUNCTION : init                                                                                          ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== Initialization function                                                                                  ===+
//+=== Check the user input parameters and convert them in appropriate types.                                   ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+================================================================================================================+
int init()
{ 
   double a;

   // Check e_period input parameter
   if( e_period < 1 )
   {
      Alert( "[ 10-ERROR  ",FILENAME," ] input parameter \"e_period\" (",e_period,") out of range: e_period >= 1 " );
      return( -1 );
   }   
   

   
   IndicatorBuffers( 1 );
   SetIndexBuffer( 0, ExtOutputBuffer );
   
   IndicatorShortName( INDICATOR_NAME );  
   SetIndexLabel( 0, INDICATOR_NAME );


   SetIndexDrawBegin( 0, e_period + 1 ); 
   
   ArrayResize( g_distance, e_period );
   ArrayInitialize( g_distance, 0.0 );
    
   return( 0 );
}

  
  
//+================================================================================================================+
//+=== FUNCTION : deinit                                                                                        ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== Custom indicator deinitialization function                                                               ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+================================================================================================================+
int deinit()
{
   return(0);
}
 
 
  
 
//+================================================================================================================+
//+=== FUNCTION : start                                                                                         ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== This callback is fired by metatrader for each tick                                                       ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+================================================================================================================+
int start()
{
   int countedBars=IndicatorCounted();


   //---- check for possible errors
   if( countedBars < 0 ) 
   {
      return(-1);
   }
   
   _computeLastNbBars( Bars - countedBars - 1 );  

   return( 0 );
}




//+================================================================================================================+
//+=== FUNCTION : _computeLastNbBars                                                                            ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== This callback is fired by metatrader for each tick                                                       ===+
//+===                                                                                                          ===+
//+=== In :                                                                                                     ===+
//+===    - lastBars : these "n" last bars must be repainted                                                    ===+
//+===                                                                                                          ===+
//+================================================================================================================+
void _computeLastNbBars( int lastBars )
{
   int    pos, count, lookback;
   static double sumCoef = 0.0;
   static double num     = 0.0;
   double fullNum, fullSumCoef;
   


   if( lastBars == 0 )
   {
      // No new bar added, it is just a tick. We use the current ("static") sumCoef and num
      g_distance[0] = 0.0;
      for( lookback = 1; lookback < e_period; lookback++ )
      {
         g_distance[0] = g_distance[0] + (Close[0] - Close[lookback])*(Close[0] - Close[lookback]);
      }
      fullNum     = num + g_distance[0]*Close[0];     
      fullSumCoef = sumCoef + g_distance[0];
            
      if( fullSumCoef != 0.0 ) ExtOutputBuffer[0] = fullNum / fullSumCoef; 
      else                     ExtOutputBuffer[0] = Close[0];
   }
   else // if( lastBars == 0 )
   {
      for( pos = lastBars; pos >= 0; pos-- )
      {      
         for( count = 1, num = 0.0, sumCoef = 0.0; count < e_period; count++ )
         {
            g_distance[count] = 0.0;
            for( lookback = 1; lookback < e_period; lookback++ )
            {
               g_distance[count] = g_distance[count] + (Close[pos+count] - Close[pos+count+lookback])*(Close[pos+count] - Close[pos+count+lookback]);
            }

            num     = num + g_distance[count]*Close[pos+count];     
            sumCoef = sumCoef + g_distance[count];
         }
         // We continue the previous loop with count=0
         count = 0;
         g_distance[count] = 0.0;
         for( lookback = 1; lookback < e_period; lookback++ )
         {
            g_distance[count] = g_distance[count] + (Close[pos+count] - Close[pos+count+lookback])*(Close[pos+count] - Close[pos+count+lookback]);
         }
         fullNum     = num + g_distance[count]*Close[pos+count];     
         fullSumCoef = sumCoef + g_distance[count];
       
      
         if( fullSumCoef != 0.0 ) ExtOutputBuffer[pos] = fullNum / fullSumCoef;
         else                     ExtOutputBuffer[pos] = Close[pos];
      }
   }
}

